import { IPublicTypeScrollable, IPublicModelSensor, IPublicModelNode, IPublicModelDropLocation, IPublicModelScrollTarget, IPublicModelLocateEvent } from '@gant-lowcode/lowcode-types';
import TreeNode from './tree-node';
import { ITreeBoard, TreeMaster } from './tree-master';
export declare class PaneController implements IPublicModelSensor, ITreeBoard, IPublicTypeScrollable {
    private pluginContext;
    private treeMaster?;
    readonly id: string;
    private indentTrack;
    private _sensorAvailable;
    /**
     * @see IPublicModelSensor
     */
    get sensorAvailable(): boolean;
    private dwell;
    /**
     * @see ITreeBoard
     */
    readonly at: string | symbol;
    private tryScrollAgain;
    private sensing;
    /**
     * @see IScrollable
     */
    get bounds(): DOMRect | null;
    private _scrollTarget?;
    /**
     * @see IScrollable
     */
    get scrollTarget(): IPublicModelScrollTarget;
    private scroller?;
    private _shell;
    constructor(at: string | symbol, treeMaster: TreeMaster);
    /** -------------------- IPublicModelSensor begin -------------------- */
    /**
     * 修正定位事件坐标信息
     * @see IPublicModelSensor.fixEvent
     */
    fixEvent(e: IPublicModelLocateEvent): IPublicModelLocateEvent;
    /**
     * 大纲树拖放定位的核心方法
     * @see IPublicModelSensor
     */
    locate(e: IPublicModelLocateEvent): IPublicModelDropLocation | undefined | null;
    /**
     * @see IPublicModelSensor
     */
    isEnter(e: IPublicModelLocateEvent): boolean;
    /**
     * @see IPublicModelSensor
     */
    deactiveSensor(): void;
    /** -------------------- IPublicModelSensor end -------------------- */
    /** -------------------- ITreeBoard begin -------------------- */
    /**
     * @see ITreeBoard
     */
    scrollToNode(treeNode: TreeNode, detail?: any, tryTimes?: number): void;
    getDropContainer(container: IPublicModelNode | null): IPublicModelNode;
    /** -------------------- ITreeBoard end -------------------- */
    /**
     * 是根据鼠标位置确定最近的插入点，处理节点前后的定位，以及插槽的特殊情况。同时，处理展开状态和递归检查，确保拖拽的合法性。
     * @param closestNode 当前最接近鼠标位置的树节点
     * @param e 拖拽事件
     * @param originalIndex 原始索引位置
     * @param originalRect 原始节点矩形区域
     * @returns
     */
    private getNear;
    /**
     * 在已展开的节点内部递归查找合适的拖放位置，处理子节点或插槽的插入逻辑，确保拖拽操作能够正确嵌套在容器或插槽中
     * @param treeNode 当前展开的父节点
     * @param e 拖拽事件
     * @returns
     */
    private drillLocate;
    purge(): void;
    /**
     * 关联DOM元素与滚动系统，初始化传感器可用状态
     * @param shell
     * @returns
     */
    mount(shell: HTMLDivElement | null): void;
    private getInsertionRect;
    private getTreeNodeRect;
    private getTreeTitleRect;
    private getTreeSlotsRect;
}
