"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
exports.__esModule = true;
exports.PaneController = void 0;
var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));
var _ricShim = _interopRequireWildcard(require("ric-shim"));
var _lowcodeUtils = require("@gant-lowcode/lowcode-utils");
var _lowcodeTypes = require("@gant-lowcode/lowcode-types");
var _indentTrack = require("../helper/indent-track");
var _dwellTimer = _interopRequireDefault(require("../helper/dwell-timer"));
function _getRequireWildcardCache(e) { if ("function" != typeof WeakMap) return null; var r = new WeakMap(), t = new WeakMap(); return (_getRequireWildcardCache = function _getRequireWildcardCache(e) { return e ? t : r; })(e); }
function _interopRequireWildcard(e, r) { if (!r && e && e.__esModule) return e; if (null === e || "object" != typeof e && "function" != typeof e) return { "default": e }; var t = _getRequireWildcardCache(r); if (t && t.has(e)) return t.get(e); var n = { __proto__: null }, a = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var u in e) if ("default" !== u && {}.hasOwnProperty.call(e, u)) { var i = a ? Object.getOwnPropertyDescriptor(e, u) : null; i && (i.get || i.set) ? Object.defineProperty(n, u, i) : n[u] = e[u]; } return n["default"] = e, t && t.set(e, n), n; }
/* eslint-disable max-len */
var PaneController = exports.PaneController = /*#__PURE__*/function () {
  function PaneController(at, treeMaster) {
    var _this = this;
    this.pluginContext = void 0;
    // 树形结构主控器
    this.treeMaster = void 0;
    this.id = (0, _lowcodeUtils.uniqueId)('outline');
    // 缩进层级跟踪器
    this.indentTrack = new _indentTrack.IndentTrack();
    this._sensorAvailable = false;
    // 悬停延迟控制器
    this.dwell = new _dwellTimer["default"](function (target, event) {
      var _this$pluginContext = _this.pluginContext,
        canvas = _this$pluginContext.canvas,
        project = _this$pluginContext.project;
      var document = project.getCurrentDocument();
      var index;
      var focus;
      var valid = true;
      if (target.hasSlots()) {
        index = null;
        focus = {
          type: 'slots'
        };
      } else {
        index = 0;
        valid = !!(document !== null && document !== void 0 && document.checkNesting(target, event.dragObject));
      }
      var _target = _this.getDropContainer(target);
      if (!_target) return;
      // 如果被拖拽的节点与计算出来的鼠标所在DOM元素节点一致，直接返回。阻止被展开
      if ((0, _lowcodeUtils.isDragNodeObject)(event.dragObject) && event.dragObject.nodes.some(function (node) {
        return (node === null || node === void 0 ? void 0 : node.id) === _target.id;
      })) return;
      canvas.createLocation({
        target: _target,
        source: _this.id,
        event: event,
        detail: {
          type: _lowcodeTypes.IPublicTypeLocationDetailType.Children,
          index: index,
          focus: focus,
          valid: valid
        }
      });
    });
    /**
     * @see ITreeBoard
     */
    this.at = void 0;
    this.tryScrollAgain = null;
    this.sensing = false;
    this._scrollTarget = void 0;
    this.scroller = void 0;
    this._shell = null;
    this.pluginContext = treeMaster.pluginContext;
    this.treeMaster = treeMaster;
    this.at = at;
    var inited = false;
    var setup = function setup() {
      var _this$treeMaster, _canvas$dragon;
      if (inited) {
        return false;
      }
      inited = true;
      (_this$treeMaster = _this.treeMaster) === null || _this$treeMaster === void 0 ? void 0 : _this$treeMaster.addBoard(_this); // 注册面板实例
      var canvas = _this.pluginContext.canvas;
      (_canvas$dragon = canvas.dragon) === null || _canvas$dragon === void 0 ? void 0 : _canvas$dragon.addSensor(_this); // 注册拖拽传感器
      _this.scroller = canvas.createScroller(_this);
    };
    setup();
  }

  /** -------------------- IPublicModelSensor begin -------------------- */

  /**
   * 修正定位事件坐标信息
   * @see IPublicModelSensor.fixEvent
   */
  var _proto = PaneController.prototype;
  _proto.fixEvent = function fixEvent(e) {
    var _e$originalEvent$view;
    // 已修正过的事件直接返回
    if (e.fixed) {
      return e;
    }
    // 判断是否来自其他文档的事件（如 iframe 中触发的事件）
    var notMyEvent = ((_e$originalEvent$view = e.originalEvent.view) === null || _e$originalEvent$view === void 0 ? void 0 : _e$originalEvent$view.document) !== document;
    if (!e.target || notMyEvent) {
      // 根据设计器坐标系重新定位目标元素
      e.target = document.elementFromPoint(e.canvasX, e.canvasY);
    }

    // 绑定当前激活的文档模型
    e.documentModel = this.pluginContext.project.getCurrentDocument();

    // 标记事件已完成修正
    e.fixed = true;
    return e;
  }

  /**
   * 大纲树拖放定位的核心方法
   * @see IPublicModelSensor
   */;
  _proto.locate = function locate(e) {
    var _this$scroller, _this$treeMaster2, _dragObject$nodes;
    // 初始化传感器状态和滚动处理
    this.sensing = true;
    (_this$scroller = this.scroller) === null || _this$scroller === void 0 ? void 0 : _this$scroller.scrolling(e); // 触发滚动计算
    var globalY = e.globalY,
      dragObject = e.dragObject;
    var nodes = dragObject === null || dragObject === void 0 ? void 0 : dragObject.nodes;
    // console.log('locate----e', e);
    var tree = (_this$treeMaster2 = this.treeMaster) === null || _this$treeMaster2 === void 0 ? void 0 : _this$treeMaster2.currentTree;
    if (!tree || !tree.root || !this._shell) {
      return null;
    }
    // 过滤可移动节点（处理组件移动限制）
    var operationalNodes = nodes === null || nodes === void 0 ? void 0 : nodes.filter(function (node) {
      var _node$componentMeta, _node$componentMeta$a;
      var onMoveHook = (_node$componentMeta = node.componentMeta) === null || _node$componentMeta === void 0 ? void 0 : (_node$componentMeta$a = _node$componentMeta.advanced.callbacks) === null || _node$componentMeta$a === void 0 ? void 0 : _node$componentMeta$a.onMoveHook;
      var canMove = onMoveHook && typeof onMoveHook === 'function' ? onMoveHook(node) : true;
      return canMove;
    }).filter(_lowcodeUtils.isDefined);
    // 如果拖拽的是 Node，且这些节点本身不允许被拖拽操作，直接退出（排除了从组件面板拖拽的情况）
    if ((0, _lowcodeUtils.isDragNodeObject)(dragObject) && (!operationalNodes || operationalNodes.length === 0)) return;
    var _this$pluginContext2 = this.pluginContext,
      project = _this$pluginContext2.project,
      canvas = _this$pluginContext2.canvas;
    var document = project.getCurrentDocument();
    // 获取节点标题区域坐标
    var originLoc = document === null || document === void 0 ? void 0 : document.dropLocation;
    var selection = document === null || document === void 0 ? void 0 : document.selection;
    // 如果存在锚定的组件，且锚定的组件不是容器，也不是插槽，不允许拖拽
    if (selection !== null && selection !== void 0 && selection.hasAnchord) {
      var anchoredNode = selection.getNodes().find(function (node) {
        return node.isAnchored;
      });
      if (!(anchoredNode !== null && anchoredNode !== void 0 && anchoredNode.isContainerNode) && !(anchoredNode !== null && anchoredNode !== void 0 && anchoredNode.isSlotNode)) {
        return;
      }
    }

    // 将全局坐标转换为大纲树节点坐标
    var pos = getPosFromEvent(e, this._shell);
    // 获取插入指示线的位置

    var dragNode = dragObject === null || dragObject === void 0 ? void 0 : (_dragObject$nodes = dragObject.nodes) === null || _dragObject$nodes === void 0 ? void 0 : _dragObject$nodes[0];
    var componentMeta = dragNode ? dragNode.componentMeta : null;

    // 处理模态组件的特殊定位逻辑
    if ((0, _lowcodeUtils.isDragNodeObject)(dragObject) && componentMeta && componentMeta.isModal && document !== null && document !== void 0 && document.focusNode) {
      console.log('1');
      var container = this.getDropContainer(document === null || document === void 0 ? void 0 : document.focusNode);
      if (!container) return null;
      return canvas.createLocation({
        target: container,
        detail: {
          type: _lowcodeTypes.IPublicTypeLocationDetailType.Children,
          index: 0,
          valid: true
        },
        source: this.id,
        event: e
      });
    }
    var irect = dragNode ? this.getTreeNodeRect(tree.getTreeNode(dragNode)) : null;

    // 处理已有定位点的缩进跟踪
    if (originLoc && (pos && pos === 'unchanged' || irect && globalY >= irect.top && globalY <= irect.bottom) && dragObject) {
      console.log('2');
      var _loc = originLoc.clone(e);
      var indented = this.indentTrack.getIndentParent(originLoc, _loc); // 缩进层级跟踪
      if (indented) {
        var parent = indented[0],
          index = indented[1];
        var _container = this.getDropContainer(parent);
        console.log('container', _container, index);
        if (checkRecursion(_container, dragObject)) {
          if (tree.getTreeNode(_container).expanded) {
            this.dwell.reset();
            if (!_container) return null;
            return canvas.createLocation({
              target: _container,
              source: this.id,
              event: e,
              detail: {
                type: _lowcodeTypes.IPublicTypeLocationDetailType.Children,
                index: index,
                valid: document === null || document === void 0 ? void 0 : document.checkNesting(_container, e.dragObject)
              }
            });
          }
          originLoc.detail.focus = {
            type: 'node',
            node: _container
          };
          if (_container) {
            // 悬停展开处理
            this.dwell.focus(_container, e);
          }
        } else {
          this.dwell.reset();
        }
        // FIXME: recreate new location
      } else if (originLoc.detail.near) {
        originLoc.detail.near = undefined;
        this.dwell.reset();
      }
      return;
    }
    this.indentTrack.reset();

    // 当检测到有效的节点位置且不是插入占位符时
    if (pos && pos !== 'unchanged') {
      console.log('3');
      // 获取目标树节点
      var closestNode = tree.getTreeNodeById(pos.nodeId);
      var notSome = operationalNodes === null || operationalNodes === void 0 ? void 0 : operationalNodes.some(function (node) {
        return node.id === (pos === null || pos === void 0 ? void 0 : pos.nodeId);
      });
      if (notSome) return;
      if (closestNode) {
        var focusSlots = pos.focusSlots;
        var _closestNode = closestNode,
          node = _closestNode.node;
        // 当拖拽的是节点类型，且存在可拖拽节点时
        if ((0, _lowcodeUtils.isDragNodeObject)(dragObject) && operationalNodes) {
          // 获取实际可拖拽的节点集合
          var newNodes = operationalNodes;

          // 从最后一个可操作节点开始遍历
          var i = (newNodes === null || newNodes === void 0 ? void 0 : newNodes.length) || 0;
          // 当前目标节点初始值
          var p = node;
          // 逆向遍历所有可操作节点
          while (i-- > 0) {
            if (newNodes[i].contains(p)) {
              p = newNodes[i].parent;
            }
          }
          if (p !== node) {
            node = p || (document === null || document === void 0 ? void 0 : document.focusNode);
            closestNode = tree.getTreeNode(node);
            focusSlots = false;
          }
        }
        if (focusSlots) {
          this.dwell.reset();
          return canvas.createLocation({
            target: node,
            source: this.id,
            event: e,
            detail: {
              type: _lowcodeTypes.IPublicTypeLocationDetailType.Children,
              index: null,
              valid: false,
              focus: {
                type: 'slots'
              }
            }
          });
        }
        if (!closestNode.isRoot()) {
          // 获取最近的投放位置
          var _loc2 = this.getNear(closestNode, e);
          this.dwell.tryFocus(_loc2);
          return _loc2;
        }
      }
    }
    console.log('4');
    var loc = this.drillLocate(tree.root, e);
    this.dwell.tryFocus(loc);
    return loc;
  }

  /**
   * @see IPublicModelSensor
   */;
  _proto.isEnter = function isEnter(e) {
    if (!this._shell) {
      return false;
    }
    var rect = this._shell.getBoundingClientRect();
    return e.globalY >= rect.top && e.globalY <= rect.bottom && e.globalX >= rect.left && e.globalX <= rect.right;
  }

  /**
   * @see IPublicModelSensor
   */;
  _proto.deactiveSensor = function deactiveSensor() {
    var _this$scroller2;
    this.sensing = false;
    (_this$scroller2 = this.scroller) === null || _this$scroller2 === void 0 ? void 0 : _this$scroller2.cancel();
    this.dwell.reset();
    this.indentTrack.reset();
  }

  /** -------------------- IPublicModelSensor end -------------------- */

  /** -------------------- ITreeBoard begin -------------------- */

  /**
   * @see ITreeBoard
   */;
  _proto.scrollToNode = function scrollToNode(treeNode, detail, tryTimes) {
    var _this2 = this;
    if (tryTimes === void 0) {
      tryTimes = 0;
    }
    if (tryTimes < 1 && this.tryScrollAgain) {
      (0, _ricShim.cancelIdleCallback)(this.tryScrollAgain);
      this.tryScrollAgain = null;
    }
    if (!this.bounds || !this.scroller || !this.scrollTarget) {
      // is a active sensor
      return;
    }
    var rect;
    if (detail && (0, _lowcodeUtils.isLocationChildrenDetail)(detail)) {
      rect = this.getInsertionRect();
    } else {
      rect = this.getTreeNodeRect(treeNode);
    }
    if (!rect) {
      if (tryTimes < 3) {
        this.tryScrollAgain = (0, _ricShim["default"])(function () {
          return _this2.scrollToNode(treeNode, detail, tryTimes + 1);
        });
      }
      return;
    }
    var _this$scrollTarget = this.scrollTarget,
      scrollHeight = _this$scrollTarget.scrollHeight,
      scrollTop = _this$scrollTarget.top;
    var _this$bounds = this.bounds,
      height = _this$bounds.height,
      top = _this$bounds.top,
      bottom = _this$bounds.bottom;
    if (rect.top < top || rect.bottom > bottom) {
      var opt = {};
      opt.top = Math.min(rect.top + rect.height / 2 + scrollTop - top - height / 2, scrollHeight - height);
      if (rect.height >= height) {
        opt.top = Math.min(scrollTop + rect.top - top, opt.top);
      }
      this.scroller.scrollTo(opt);
    }
    // make tail scroll be sure
    if (tryTimes < 4) {
      this.tryScrollAgain = (0, _ricShim["default"])(function () {
        return _this2.scrollToNode(treeNode, detail, 4);
      });
    }
  };
  _proto.getDropContainer = function getDropContainer(container) {
    var _canvas$dragon2, _project$currentDocum;
    var _this$pluginContext3 = this.pluginContext,
      canvas = _this$pluginContext3.canvas,
      project = _this$pluginContext3.project;
    return (_canvas$dragon2 = canvas.dragon) === null || _canvas$dragon2 === void 0 ? void 0 : _canvas$dragon2.getAnchorDropContainer((_project$currentDocum = project.currentDocument) === null || _project$currentDocum === void 0 ? void 0 : _project$currentDocum.selection, container);
  }

  /** -------------------- ITreeBoard end -------------------- */

  /**
   * 是根据鼠标位置确定最近的插入点，处理节点前后的定位，以及插槽的特殊情况。同时，处理展开状态和递归检查，确保拖拽的合法性。
   * @param closestNode 当前最接近鼠标位置的树节点
   * @param e 拖拽事件
   * @param originalIndex 原始索引位置
   * @param originalRect 原始节点矩形区域
   * @returns
   */;
  _proto.getNear = function getNear(closestNode, e, originalIndex, originalRect) {
    var _this$pluginContext4 = this.pluginContext,
      canvas = _this$pluginContext4.canvas,
      project = _this$pluginContext4.project;
    var document = project.getCurrentDocument();
    var globalY = e.globalY,
      dragObject = e.dragObject;
    if (!dragObject) {
      return null;
    }
    // TODO: check dragObject is anyData
    var node = closestNode.node,
      expanded = closestNode.expanded;
    var rect = originalRect;
    if (!rect) {
      // 获取被拖拽节点的尺寸坐标信息
      rect = this.getTreeNodeRect(closestNode);
      if (!rect) {
        return null;
      }
    }
    var index = originalIndex;
    if (index == null) {
      index = node.index;
    }

    // 插槽根节点特殊处理
    if (expanded) {
      // 是个插槽根节点
      if (!closestNode.isContainer() && !closestNode.hasSlots()) {
        var _container2 = this.getDropContainer(node.parent);
        if (!_container2) return;
        // 如果是插槽根节点，进行替换
        return canvas.createLocation({
          target: _container2,
          source: this.id,
          event: e,
          detail: {
            type: _lowcodeTypes.IPublicTypeLocationDetailType.Children,
            index: null,
            near: {
              node: node,
              pos: 'replace'
            },
            // 特殊标记替换操作
            valid: true // TODO: future validation the slot limit
          }
        });
      }
      var loc1 = this.drillLocate(closestNode, e);
      if (loc1) {
        return loc1;
      }
      var _container3 = this.getDropContainer(node.parent);
      if (!_container3) return;
      return canvas.createLocation({
        target: _container3,
        source: this.id,
        event: e,
        detail: {
          type: _lowcodeTypes.IPublicTypeLocationDetailType.Children,
          index: null,
          valid: false,
          focus: {
            type: 'slots'
          }
        }
      });
    }
    var focusNode;
    // focus
    if (!expanded && (closestNode.isContainer() || closestNode.hasSlots())) {
      focusNode = node;
    }

    // 位置判断逻辑
    var titleRect = this.getTreeTitleRect(closestNode) || rect;
    // 节点上半区：before 定位
    if (globalY < titleRect.top + titleRect.height / 2) {
      var _container4 = this.getDropContainer(node.parent);
      if (!_container4) return;
      return canvas.createLocation({
        target: _container4,
        source: this.id,
        event: e,
        detail: {
          type: _lowcodeTypes.IPublicTypeLocationDetailType.Children,
          index: index,
          valid: document === null || document === void 0 ? void 0 : document.checkNesting(_container4, dragObject),
          near: {
            node: node,
            pos: 'before'
          },
          // 前向定位
          focus: checkRecursion(focusNode, dragObject) ? {
            type: 'node',
            node: focusNode
          } : undefined
        }
      });
    }
    if (globalY > titleRect.bottom) {
      focusNode = undefined;
    }
    if (expanded) {
      // drill
      var loc = this.drillLocate(closestNode, e);
      if (loc) {
        return loc;
      }
    }
    var container = this.getDropContainer(node.parent);
    if (!container) return;
    // 节点下半区：after 定位
    return canvas.createLocation({
      target: container,
      source: this.id,
      event: e,
      detail: {
        type: _lowcodeTypes.IPublicTypeLocationDetailType.Children,
        index: (index || 0) + 1,
        valid: document === null || document === void 0 ? void 0 : document.checkNesting(container, dragObject),
        near: {
          node: node,
          pos: 'after'
        },
        focus: checkRecursion(focusNode, dragObject) ? {
          type: 'node',
          node: focusNode
        } : undefined
      }
    });
  }

  /**
   * 在已展开的节点内部递归查找合适的拖放位置，处理子节点或插槽的插入逻辑，确保拖拽操作能够正确嵌套在容器或插槽中
   * @param treeNode 当前展开的父节点
   * @param e 拖拽事件
   * @returns
   */;
  _proto.drillLocate = function drillLocate(closestNode, e) {
    var _this$pluginContext5 = this.pluginContext,
      canvas = _this$pluginContext5.canvas,
      project = _this$pluginContext5.project;
    var document = project.getCurrentDocument();
    var dragObject = e.dragObject,
      globalY = e.globalY;
    if (!dragObject) {
      return null;
    }
    // 检查是否允许拖拽到当前容器（防⽌循环嵌套）
    if (!checkRecursion(closestNode.node, dragObject)) {
      return null;
    }
    if ((0, _lowcodeUtils.isDragAnyObject)(dragObject)) {
      // TODO: future
      return null;
    }
    var container = closestNode.node;
    container = this.getDropContainer(container);
    if (!container) return null;
    var detail = {
      type: _lowcodeTypes.IPublicTypeLocationDetailType.Children
    };
    var locationData = {
      target: container,
      detail: detail,
      source: this.id,
      event: e
    };
    console.log('---drill locationData', closestNode, locationData);
    var isSlotContainer = closestNode.hasSlots();
    var isContainer = closestNode.isContainer();

    // 处理插槽容器的特殊定位
    if (container.isSlotNode && !closestNode.expanded) {
      // 未展开，直接定位到内部第一个节点
      if (isSlotContainer) {
        detail.index = null;
        detail.focus = {
          type: 'slots'
        };
        detail.valid = false;
      } else {
        detail.index = 0;
        detail.valid = document === null || document === void 0 ? void 0 : document.checkNesting(container, dragObject);
      }
    }
    var items = null;
    var slotsRect;
    var focusSlots = false;
    // isSlotContainer
    if (isSlotContainer) {
      slotsRect = this.getTreeSlotsRect(closestNode); // 获取插槽区域坐标
      if (slotsRect) {
        if (globalY <= slotsRect.bottom) {
          focusSlots = true; // 标记插槽聚焦状态
          items = closestNode.slots;
        } else if (!isContainer) {
          // 不在 slots 范围，又不是 container 的情况，高亮 slots 区
          detail.index = null;
          detail.focus = {
            type: 'slots'
          };
          detail.valid = false;
          return canvas.createLocation(locationData);
        }
      }
    }
    if (!items && isContainer) {
      items = closestNode.children;
    }
    if (!items) {
      return null;
    }
    var l = items.length;
    var index = 0;
    var before = l < 1;
    var current;
    var currentIndex = index;
    for (; index < l; index++) {
      current = items[index];
      currentIndex = index;
      var rect = this.getTreeNodeRect(current);
      if (!rect) {
        continue;
      }

      // rect
      if (globalY < rect.top) {
        before = true;
        break;
      }
      if (globalY > rect.bottom) {
        continue;
      }
      var loc = this.getNear(current, e, index, rect);
      if (loc) {
        return loc;
      }
    }
    if (focusSlots) {
      detail.focus = {
        type: 'slots'
      };
      detail.valid = false;
      detail.index = null;
    } else {
      if (current) {
        detail.index = before ? currentIndex : currentIndex + 1;
        detail.near = {
          node: current.node,
          pos: before ? 'before' : 'after'
        };
      } else {
        detail.index = l;
      }
      detail.valid = document === null || document === void 0 ? void 0 : document.checkNesting(container, dragObject);
    }
    return canvas.createLocation(locationData);
  };
  _proto.purge = function purge() {
    var _canvas$dragon3, _this$treeMaster3;
    var canvas = this.pluginContext.canvas;
    (_canvas$dragon3 = canvas.dragon) === null || _canvas$dragon3 === void 0 ? void 0 : _canvas$dragon3.removeSensor(this);
    (_this$treeMaster3 = this.treeMaster) === null || _this$treeMaster3 === void 0 ? void 0 : _this$treeMaster3.removeBoard(this);
  }

  /**
   * 关联DOM元素与滚动系统，初始化传感器可用状态
   * @param shell
   * @returns
   */;
  _proto.mount = function mount(shell) {
    var _this3 = this;
    if (this._shell === shell) {
      return;
    }
    this._shell = shell;
    var _this$pluginContext6 = this.pluginContext,
      canvas = _this$pluginContext6.canvas,
      project = _this$pluginContext6.project;
    if (shell) {
      var _project$currentDocum2, _this$treeMaster4;
      this._scrollTarget = canvas.createScrollTarget(shell);
      this._sensorAvailable = true;

      // check if there is current selection and scroll to it
      var selection = (_project$currentDocum2 = project.currentDocument) === null || _project$currentDocum2 === void 0 ? void 0 : _project$currentDocum2.selection;
      var topNodes = selection === null || selection === void 0 ? void 0 : selection.getTopNodes(true);
      var tree = (_this$treeMaster4 = this.treeMaster) === null || _this$treeMaster4 === void 0 ? void 0 : _this$treeMaster4.currentTree;
      if (topNodes && topNodes[0] && tree) {
        var treeNode = tree.getTreeNodeById(topNodes[0].id);
        if (treeNode) {
          // at this moment, it is possible that pane is not ready yet, so
          // put ui related operations to the next loop
          setTimeout(function () {
            tree.setNodeSelected(treeNode.nodeId);
            _this3.scrollToNode(treeNode, null, 4);
          }, 0);
        }
      }
    } else {
      this._scrollTarget = undefined;
      this._sensorAvailable = false;
    }
  };
  _proto.getInsertionRect = function getInsertionRect() {
    var _this$_shell$querySel;
    if (!this._shell) {
      return undefined;
    }
    return (_this$_shell$querySel = this._shell.querySelector('.insertion')) === null || _this$_shell$querySel === void 0 ? void 0 : _this$_shell$querySel.getBoundingClientRect();
  };
  _proto.getTreeNodeRect = function getTreeNodeRect(treeNode) {
    var _this$_shell$querySel2;
    if (!this._shell) {
      return undefined;
    }
    return (_this$_shell$querySel2 = this._shell.querySelector(".tree-node[data-id=\"" + treeNode.nodeId + "\"]")) === null || _this$_shell$querySel2 === void 0 ? void 0 : _this$_shell$querySel2.getBoundingClientRect();
  };
  _proto.getTreeTitleRect = function getTreeTitleRect(treeNode) {
    var _this$_shell$querySel3;
    if (!this._shell) {
      return undefined;
    }
    return (_this$_shell$querySel3 = this._shell.querySelector(".tree-node-title[data-id=\"" + treeNode.nodeId + "\"]")) === null || _this$_shell$querySel3 === void 0 ? void 0 : _this$_shell$querySel3.getBoundingClientRect();
  };
  _proto.getTreeSlotsRect = function getTreeSlotsRect(treeNode) {
    var _this$_shell$querySel4;
    if (!this._shell) {
      return undefined;
    }
    return (_this$_shell$querySel4 = this._shell.querySelector(".tree-node-slots[data-id=\"" + treeNode.nodeId + "\"]")) === null || _this$_shell$querySel4 === void 0 ? void 0 : _this$_shell$querySel4.getBoundingClientRect();
  };
  return (0, _createClass2["default"])(PaneController, [{
    key: "sensorAvailable",
    get:
    /**
     * @see IPublicModelSensor
     */
    function get() {
      return this._sensorAvailable;
    }
  }, {
    key: "bounds",
    get:
    /**
     * @see IScrollable
     */
    function get() {
      if (!this._shell) {
        return null;
      }
      return this._shell.getBoundingClientRect();
    }
  }, {
    key: "scrollTarget",
    get:
    /**
     * @see IScrollable
     */
    function get() {
      return this._scrollTarget;
    }
  }]);
}();
/**
 * 用于 防止循环嵌套 的递归检查
 * @param parent 目标父节点
 * @param dragObject 拖拽对象
 * @returns
 */
function checkRecursion(parent, dragObject) {
  // 无父节点时直接返回不可拖拽
  if (!parent) {
    return false;
  }
  // 仅当拖拽的是节点对象时进行检查
  if ((0, _lowcodeUtils.isDragNodeObject)(dragObject)) {
    var nodes = dragObject.nodes;
    // 核心检查逻辑：确保被拖拽的节点不包含目标父节点
    if (nodes.some(function (node) {
      return node.contains(parent);
    })) {
      return false;
    }
  }
  // 非节点拖拽直接通过
  return true;
}

/**
 * 获取到当前鼠标所在位置的DOM元素的node id，以及是否存在插槽区域
 * @param param0
 * @param shell
 * @returns
 */
function getPosFromEvent(_ref,
// 拖拽定位事件
shell // 大纲树容器元素
) {
  var target = _ref.target,
    dragObject = _ref.dragObject;
  // 过滤非本树容器内的事件
  if (!target || !shell.contains(target)) {
    return null;
  }
  // 识别插入指示线区域
  if (target.matches('.insertion')) {
    return 'unchanged';
  }
  // 通过DOM查询获取节点ID
  var closest = target.closest('[data-id]');
  if (!closest || !shell.contains(closest)) {
    return null;
  }

  // 提取DOM上的节点ID
  var nodeId = closest.dataset.id;
  if ((0, _lowcodeUtils.isDragNodeObject)(dragObject) && dragObject.nodes.some(function (node) {
    return (node === null || node === void 0 ? void 0 : node.id) === nodeId;
  })) {
    return 'unchanged';
  }
  return {
    focusSlots: closest.matches('.tree-node-slots'),
    // 判断是否在插槽区域
    nodeId: nodeId
  };
}